#include <vector>
#include <iostream>
#include <fstream>

#include "xmltio.hpp"
#include "XOPExampleData.hpp"

using namespace std;
using namespace xmltio;

// define an xml document template to be contructed
const string templateExample("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\" ?><EXAMPLE><ELEMENT attr1=\"false\" attr2=\"1.0\"/><ELEMENT2><SUBELEMENT></SUBELEMENT></ELEMENT2></EXAMPLE>");

int main(int argc, char* argv[]) {

	// HACKHACKHACK
	// Temporary driver for XOP Example Data class
	XOPExampleDataPtr package(new XOPExampleData());

	cout << "XOP Document: " << endl << package->getDocument() << endl;

	cout << "Lena (binary part): " << endl;

	XOPAttachmentPtr lena = package->getContentWithId("lena.jpg");

	XOPAttachment::iterator it;

	// write attachment to binary file
	if (lena->size() > 0) {
		ofstream os("lena_copy.jpg",ios::out | ios::binary);
		os.write(reinterpret_cast<char*>(&(*lena)[0]), lena->size());
		os.close();
		cout << "Data contained in lena.jpg has been written to lena_copy.jpg" << endl;
	}

	// exemplary root-location object
	// constructs XML document tree from given template
	Location doc(templateExample, "/EXAMPLE");

	// set some text value of an element node
	// exemplary use of a new sub-location with a relative XPath
	Location l(doc, "ELEMENT2/SUBELEMENT");
	const string input("some text");
	// direct assignmemt of text to node
	l = input;

	// change attribute value
	doc[XPath("ELEMENT/@attr1")]="true";

	// add element
	Location n("<ELEMENT3>18</ELEMENT3>","/ELEMENT3");

	// location now points at ELEMENT3
	doc = doc.add(n);

	// add attribute to ELEMENT3 node
	doc["attr"]="new";

	// print whole document
	cout << "Resulting document (pretty printed):" << endl;
	cout << doc.getDocumentText(true) << endl;

	// reset location to root element
	doc = Location(doc,"/EXAMPLE");
	// access parts
	cout << "Name of root element is: " << doc.getName() << endl;
	cout << "Value of root element is: " << doc << endl;
	cout << "Name of sub node ELEMENT2: " << doc["ELEMENT2"].getName() << endl;
	cout << "Value of sub node ELEMENT2: " << doc["ELEMENT2"] << endl;
	cout << "Value of attribute attr2: " << extract<int>(doc[XPath("ELEMENT/@attr2")]) << endl;
	cout << "Access value of node SUBELEMENT in arbitrary location: " << doc[XPath("//SUBELEMENT")] << endl;
	cout << "Access text value of node SUBELEMENT in arbitrary location: " << doc[XPath("//SUBELEMENT")].getTextValue() << endl;
	cout << "Access int value of node ELEMENT3 in arbitrary location: " << extract<int>(doc[XPath("//ELEMENT3")]) << endl;

	return 0;
}
