/* -*- mode: C++; tab-width: 8; c-basic-offset: 8; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 *
 * @file	Template.hpp
 * @author	Ingo Luetkebohle (iluetkeb@TechFak.Uni-Bielefeld.de)
 * @date	started at 2004/08/03
 * @version	$Id: Template.hpp 4247 2015-01-11 15:07:52Z rhaschke $
 *
 **/

#ifndef __XMLTIO_TEMPLATE_HPP
#define __XMLTIO_TEMPLATE_HPP

#include "XPath.hpp"
#include "Location.hpp"

#include <string>

namespace xmltio {
/** \brief XML template information.
 *
 * Specialize for user defined types.
 *
 */
template<typename T>
struct template_traits {
	/// a full XML document to serve as the template
	static const std::string tmpl;
	/// anchor point: where the information is inside the XML
	static const XPath anchor;
};
template<typename T>
Location find_anchor(const std::string& xml)
{
	return Location(xml, template_traits<T>::anchor);
}
template<typename T>
Location create_template()
{
	return find_anchor<T>(template_traits<T>::tmpl);
}
template<typename T>
Location create_template(const T& e)
{
	Location l = create_template<T>();
	l = e;
	return l;
}

template<typename T>
std::string to_xml(const T& e)
{
	Location l(create_template<T>());
	l = e;
	return dom2string(l.getDocument());
}

}

#define TEMPLATE_TRAITS(TYPE, TMPL, ANCHOR)                             \
	namespace xmltio { \
	template<> const std::string template_traits<TYPE>::tmpl(TMPL); \
	template<> const XPath template_traits<TYPE>::anchor(ANCHOR); \
	}

#endif //  __XMLTIO_TEMPLATE_HPP
