/* -*- mode: C++; tab-width: 8; c-basic-offset: 8; c-file-offsets: ((substatement-open . 0)) -*- */

#include "exception.hpp"
#include "XPath.hpp"
#include "str.hpp"

#include <sstream>

using namespace std;

namespace xmltio {
using XERCES_CPP_NAMESPACE::DOMException;

string code2string(int code)
{
	switch(code) {
	case DOMException::INDEX_SIZE_ERR:
		return "index size error";
	case DOMException::DOMSTRING_SIZE_ERR:
		return "domstring size error";
	case DOMException::HIERARCHY_REQUEST_ERR:
		return "hierarchy request error";
	case DOMException::WRONG_DOCUMENT_ERR:
		return "wrong document";
	case DOMException::INVALID_CHARACTER_ERR:
		return "invalid character";
	case DOMException::NO_DATA_ALLOWED_ERR:
		return "no data allowed here";
	case DOMException::NOT_FOUND_ERR:
		return "not found";
	case DOMException::NOT_SUPPORTED_ERR:
		return "operation not supported";
	case DOMException::NO_MODIFICATION_ALLOWED_ERR:
		return "no modification allowed";
	case DOMException::INUSE_ATTRIBUTE_ERR:
		return "attribute already in use";
	case DOMException::INVALID_STATE_ERR:
		return "invalid state";
	case DOMException::SYNTAX_ERR:
		return "syntax error";
	case DOMException::INVALID_MODIFICATION_ERR:
		return "invalid modification";
	case DOMException::NAMESPACE_ERR:
		return "namespace error";
	case DOMException::INVALID_ACCESS_ERR:
		return "invalid access";
	case DOMException::VALIDATION_ERR:
		return "validation error";
	default:
		std::ostringstream buf;
		buf << "unrecognized DOM error: "
		    << code;
		return buf.str();
	}
}
XMLTIO_Exception::XMLTIO_Exception(const DOMException& ex)
{
	ostringstream msg;
	msg << "DOMException[";
	if(ex.msg) {
		msg << ex.msg;
	} else {
		msg << code2string(ex.code);
	}
	msg << "]";

	msg_ = msg.str();
}
XMLTIO_Exception::XMLTIO_Exception(const string& msg,
                                   const DOMException& ex) :
	msg_(msg)
{
	msg_.append(ex.msg ? (const char*)NX(ex.msg) :
	            code2string(ex.code));
}

NotFoundException::NotFoundException(const std::string& msg,
                                     const XPath& xpath) :
	XMLTIO_Exception(msg + " xpath: " + xpath.getXPath())
{
}
}
