/* -*- mode: C++; tab-width: 8; c-basic-offset: 8; c-file-offsets: ((substatement-open . 0)) -*- */
/**
 * @file	str.hpp
 * @author	Ingo Luetkebohle (iluetkeb@TechFak.Uni-Bielefeld.de)
 * @date	started at 2004/06/18
 * @version	$Id: str.hpp 4251 2015-01-14 21:35:25Z rhaschke $
 *
 * @brief       string conversion functions for DOM usage
 *
 */

#ifndef __XMLTIO_STR_HPP
#define __XMLTIO_STR_HPP

#include <string>
#include <sstream>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/util/XMLString.hpp>
#include "xmltio_config.hpp"

namespace xmltio {
/// serialize DOM tree to string
std::string XMLTIO_API dom2string(const XERCES_CPP_NAMESPACE::DOMNode*);
std::string XMLTIO_API dom2string(const XERCES_CPP_NAMESPACE::DOMNode*, bool pprint);

class TIODocument;
std::string XMLTIO_API dom2string(const TIODocument& n);
std::string XMLTIO_API dom2string(const TIODocument& n, bool pprint);

/// convert std::string to XMLString for Xerces-C
class XMLTIO_API X {
private:
	XMLCh* xmlch_;

	// forbidden, not implemented
	X(const X&);
public:
	X(const std::string& s);
	~X();

	operator XMLCh*() { return xmlch_; };
	operator const XMLCh*() const { return xmlch_; };
};
/// convert Xerces-C XMLCh* to const char*
class XMLTIO_API NX {
private:
	char* ch_;

	// forbidden, not implemented
	NX(const NX&);
public:
	NX(const XMLCh* s);
	~NX();

	operator const char*() const { return ch_; };
	operator std::string() const { return std::string(ch_); }
};


} // namespace xmltio

#endif // __XMLTIO_STR_HPP
